-----------------------------------------------------------------------------------------------
-- Client Lua Script for SpellSwap
-- Copyright (c) NCsoft. All rights reserved

-- v2.0 Changelog
-- Added 4 buttons ("Swaps")
-- Color coded buttons and UI
-- Changed Button border to distinguish from QCGR

-- v2.1 Changelog
-- Fixed deadspot by adding IgnoreMouse attribute to Window
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SpellSwap Module Definition
-----------------------------------------------------------------------------------------------
local SpellSwap = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local InactiveSprite = "BK3:UI_BK3_Holo_Framing_2_Blocker"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SpellSwap:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self:SetDefaults()
    return o
end

function SpellSwap:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SpellSwap OnLoad
-----------------------------------------------------------------------------------------------
function SpellSwap:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SpellSwap.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SpellSwap OnDocLoaded
-----------------------------------------------------------------------------------------------
function SpellSwap:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Window", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(true, true)
		
		if self.vars1.l ~= nil then
			self.wndMain:FindChild("Swap1Window"):SetAnchorOffsets(self.vars1.l,self.vars1.t,self.vars1.r,self.vars1.b)
		end
		if self.vars2.l ~= nil then
			self.wndMain:FindChild("Swap2Window"):SetAnchorOffsets(self.vars2.l,self.vars2.t,self.vars2.r,self.vars2.b)
		end
		if self.vars3.l ~= nil then
			self.wndMain:FindChild("Swap3Window"):SetAnchorOffsets(self.vars3.l,self.vars3.t,self.vars3.r,self.vars3.b)
		end
		if self.vars4.l ~= nil then
			self.wndMain:FindChild("Swap4Window"):SetAnchorOffsets(self.vars4.l,self.vars4.t,self.vars4.r,self.vars4.b)
		end
		
		self:SetIconLock(1, self.vars1.bLock)
		self:SetIconLock(2, self.vars2.bLock)
		self:SetIconLock(3, self.vars3.bLock)
		self:SetIconLock(4, self.vars4.bLock)
		
		self:SetIconHide(1, self.vars1.bHide)
		self:SetIconHide(2, self.vars2.bHide)
		self:SetIconHide(3, self.vars3.bHide)
		self:SetIconHide(4, self.vars4.bHide)
		
		self:SkillButtonUpdate(1)
		self:SkillButtonUpdate(2)
		self:SkillButtonUpdate(3)
		self:SkillButtonUpdate(4)
		
		Apollo.RegisterEventHandler("AbilityBookChange","OnAbilityChange",self)
		Apollo.RegisterTimerHandler("SpellSwap_DelayTimer","OnDelayTimer",self)
		Apollo.RegisterSlashCommand("ss", "OnSlashCommand", self)
		Apollo.RegisterSlashCommand("ss1", "HandleSwap1", self)
		Apollo.RegisterSlashCommand("ss2", "HandleSwap2", self)
		Apollo.RegisterSlashCommand("ss3", "HandleSwap3", self)
		Apollo.RegisterSlashCommand("ss4", "HandleSwap4", self)
		
		-- Handle spell changes
		Apollo.RegisterSlashCommand("ss[1,1]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[1,2]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[2,1]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[2,2]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[3,1]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[3,2]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[4,1]", "AddSpell", self)
		Apollo.RegisterSlashCommand("ss[4,2]", "AddSpell", self)
	end
end

-----------------------------------------------------------------------------------------------
-- SpellSwap Functions
-----------------------------------------------------------------------------------------------

---------- Slash command functions ----------

function SpellSwap:OnSlashCommand(strCmd, strArg)
	if strArg == "lock" then
		self.vars1.bLock = 1
		self.vars2.bLock = 1
		self.vars3.bLock = 1
		self.vars4.bLock = 1
		
		self:SetIconLock(1, self.vars1.bLock)
		self:SetIconLock(2, self.vars2.bLock)
		self:SetIconLock(3, self.vars3.bLock)
		self:SetIconLock(4, self.vars4.bLock)
	elseif strArg == "unlock" then
		self.vars1.bLock = 0
		self.vars2.bLock = 0
		self.vars3.bLock = 0
		self.vars4.bLock = 0
		
		self:SetIconLock(1, self.vars1.bLock)
		self:SetIconLock(2, self.vars2.bLock)
		self:SetIconLock(3, self.vars3.bLock)
		self:SetIconLock(4, self.vars4.bLock)
	elseif strArg == "hide" then
		self.vars1.bHide = 1
		self.vars2.bHide = 1
		self.vars3.bHide = 1
		self.vars4.bHide = 1
		
		self:SetIconHide(1, self.vars1.bHide)
		self:SetIconHide(2, self.vars2.bHide)
		self:SetIconHide(3, self.vars3.bHide)
		self:SetIconHide(4, self.vars4.bHide)
	elseif strArg == "show" or strArg == "unhide" then
		self.vars1.bHide = 0
		self.vars2.bHide = 0
		self.vars3.bHide = 0
		self.vars4.bHide = 0
		
		self:SetIconHide(1, self.vars1.bHide)
		self:SetIconHide(2, self.vars2.bHide)
		self:SetIconHide(3, self.vars3.bHide)
		self:SetIconHide(4, self.vars4.bHide)	
	elseif strArg == "reset" then
		self:SetDefaults()
		self.fromReset = true
		self:SkillButtonUpdate(1)
		self:SkillButtonUpdate(2)
		self:SkillButtonUpdate(3)
		self:SkillButtonUpdate(4)
	else
		self:ShowOptionsMenu()
	end		
end

function SpellSwap:HandleSwap1(strCmd, strArg)
	if strArg == "lock" then
		self.vars1.bLock = 1
		self:SetIconLock(1, self.vars1.bLock)
	elseif strArg == "unlock" then
		self.vars1.bLock = 0
		self:SetIconLock(1, self.vars1.bLock)
	elseif strArg == "hide" then
		self.vars1.bHide = 1
		self:SetIconHide(1, self.vars1.bHide)
	elseif strArg == "show" or strArg == "unhide" then
		self.vars1.bHide = 0
		self:SetIconHide(1, self.vars1.bHide)
	elseif strArg == "toggle" then
		self:ToggleSpellSwap(1)
	else
		Print("Not a valid command for Swap 1.")
	end
end

function SpellSwap:HandleSwap2(strCmd, strArg)
	if strArg == "lock" then
		self.vars2.bLock = 1
		self:SetIconLock(2, self.vars2.bLock)
	elseif strArg == "unlock" then
		self.vars2.bLock = 0
		self:SetIconLock(2, self.vars2.bLock)
	elseif strArg == "hide" then
		self.vars2.bHide = 1
		self:SetIconHide(2, self.vars2.bHide)
	elseif strArg == "show" or strArg == "unhide" then
		self.vars2.bHide = 0
		self:SetIconHide(2, self.vars2.bHide)
	elseif strArg == "toggle" then
		self:ToggleSpellSwap(2)
	else
		Print("Not a valid command for Swap 2.")
	end
end

function SpellSwap:HandleSwap3(strCmd, strArg)
	if strArg == "lock" then
		self.vars3.bLock = 1
		self:SetIconLock(3, self.vars3.bLock)
	elseif strArg == "unlock" then
		self.vars3.bLock = 0
		self:SetIconLock(3, self.vars3.bLock)
	elseif strArg == "hide" then
		self.vars3.bHide = 1
		self:SetIconHide(3, self.vars3.bHide)
	elseif strArg == "show" or strArg == "unhide" then
		self.vars3.bHide = 0
		self:SetIconHide(3, self.vars3.bHide)
	elseif strArg == "toggle" then
		self:ToggleSpellSwap(3)
	else
		Print("Not a valid command for Swap 3.")
	end
end

function SpellSwap:HandleSwap4(strCmd, strArg)
	if strArg == "lock" then
		self.vars4.bLock = 1
		self:SetIconLock(4, self.vars4.bLock)
	elseif strArg == "unlock" then
		self.vars4.bLock = 0
		self:SetIconLock(4, self.vars4.bLock)
	elseif strArg == "hide" then
		self.vars4.bHide = 1
		self:SetIconHide(4, self.vars4.bHide)
	elseif strArg == "show" or strArg == "unhide" then
		self.vars4.bHide = 0
		self:SetIconHide(4, self.vars4.bHide)
	elseif strArg == "toggle" then
		self:ToggleSpellSwap(4)
	else
		Print("Not a valid command for Swap 4.")
	end
end

function SpellSwap:AddSpell(strCmd, strArg)
	local swap = strCmd:sub(4,4)
	local spell = strCmd:sub(6,6)
	local spellName = strArg
	
	if spell == "1" then
		self:AddSpell1(swap, spell, spellName)
	else
		self:AddSpell2(swap, spell, spellName)
	end
end

function SpellSwap:AddSpell1(swap, spell, spellName)
	local spell1 = spellName
	local spell1Id = self:GetSpellId(spell1)
	
	if spell1Id == nil then
		Print("[WARNING - SpellSwap]: Spell '" .. spell1 .. "' could not be found.")
	elseif spell1 == self.vars1.spell2 then
		Print("[WARNING - SpellSwap]: Spells cannot be the same.")
	else
		if swap == "4" then
			self.vars4.spell1 = spell1
			self.vars4.spell1Id = spell1Id
			self.vars4.spell1Icon = self:GetSpellIconByName(self.vars4.spell1)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap4"):FindChild("Spell1TextBox"):SetText(self.vars4.spell1)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(4)
		elseif swap == "3" then
			self.vars3.spell1 = spell1
			self.vars3.spell1Id = spell1Id
			self.vars3.spell1Icon = self:GetSpellIconByName(self.vars3.spell1)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap3"):FindChild("Spell1TextBox"):SetText(self.vars3.spell1)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(3)
		elseif swap == "2" then
			self.vars2.spell1 = spell1
			self.vars2.spell1Id = spell1Id
			self.vars2.spell1Icon = self:GetSpellIconByName(self.vars2.spell1)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap2"):FindChild("Spell1TextBox"):SetText(self.vars2.spell1)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(2)
		else
			self.vars1.spell1 = spell1
			self.vars1.spell1Id = spell1Id
			self.vars1.spell1Icon = self:GetSpellIconByName(self.vars1.spell1)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap1"):FindChild("Spell1TextBox"):SetText(self.vars1.spell1)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(1)
		end
	end
end

function SpellSwap:AddSpell2(swap, spell, spellName)
	local spell2 = spellName
	local spell2Id = self:GetSpellId(spell2)
	
	if spell2Id == nil then
		Print("[WARNING - SpellSwap]: Spell '" .. spell2 .. "' could not be found.")
	elseif spell2 == self.vars1.spell2 then
		Print("[WARNING - SpellSwap]: Spells cannot be the same.")
	else
		if swap == "4" then
			self.vars4.spell2 = spell2
			self.vars4.spell2Id = spell2Id
			self.vars4.spell2Icon = self:GetSpellIconByName(self.vars4.spell2)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap4"):FindChild("Spell1TextBox"):SetText(self.vars4.spell2)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(4)
		elseif swap == "3" then
			self.vars3.spell2 = spell2
			self.vars3.spell2Id = spell2Id
			self.vars3.spell2Icon = self:GetSpellIconByName(self.vars3.spell2)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap3"):FindChild("Spell1TextBox"):SetText(self.vars3.spell2)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(3)
		elseif swap == "2" then
			self.vars2.spell2 = spell2
			self.vars2.spell2Id = spell2Id
			self.vars2.spell2Icon = self:GetSpellIconByName(self.vars2.spell2)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap2"):FindChild("Spell1TextBox"):SetText(self.vars2.spell2)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(2)
		else
			self.vars1.spell2 = spell2
			self.vars1.spell2Id = spell2Id
			self.vars1.spell2Icon = self:GetSpellIconByName(self.vars1.spell2)
			if self.wndOptions then
				self.wndOptions:FindChild("Swap1"):FindChild("Spell1TextBox"):SetText(self.vars1.spell1)
			end
			self.fromSlashCommand = true
			self:SkillButtonUpdate(1)
		end
	end
end

---------- Options Menu functions ----------

function SpellSwap:ShowOptionsMenu()
	if self.wndOptions then
		self:CloseOptionsMenu()
	end
	self.wndOptions = Apollo.LoadForm(self.xmlDoc,"OptionsMenu",nil,self)
	self:SetIconLock(1, self.vars1.bLock)
	self:SetIconLock(2, self.vars2.bLock)
	self:SetIconLock(3, self.vars3.bLock)
	self:SetIconLock(4, self.vars4.bLock)
	
	self:SetIconHide(1, self.vars1.bHide)
	self:SetIconHide(2, self.vars2.bHide)
	self:SetIconHide(3, self.vars3.bHide)
	self:SetIconHide(4, self.vars4.bHide)
	
	self:SetSpellTextBoxes()
end

function SpellSwap:CloseOptionsMenu()
	if not self.wndOptions then
		return
	end
	self.wndOptions:Destroy()
	self.wndOptions = nil
end

function SpellSwap:SetIconLock(swap, locked)
	if self.wndMain then
		if swap == 4 then
			self.wndMain:FindChild("Swap4Window"):SetStyle("Moveable",locked == 0)
			self.wndMain:FindChild("Swap4Window"):SetStyle("Sizable",locked  == 0)
			if locked == 1 then
				self.wndMain:FindChild("Swap4Window"):SetStyle("IgnoreMouse", true)
				self.wndMain:FindChild("Swap4Window"):SetSprite(nil)
				self.wndMain:FindChild("Swap4Window"):SetTooltip("")
			else
				self.wndMain:FindChild("Swap4Window"):SetStyle("IgnoreMouse", false)
				self.wndMain:FindChild("Swap4Window"):SetSprite("AbilitiesSprites:spr_FrameSelection")
				self.wndMain:FindChild("Swap4Window"):SetTooltip("Swap 4 - Move me!")
			end
		elseif swap == 3 then
			self.wndMain:FindChild("Swap3Window"):SetStyle("Moveable",locked == 0)
			self.wndMain:FindChild("Swap3Window"):SetStyle("Sizable",locked  == 0)
			if locked == 1 then
				self.wndMain:FindChild("Swap3Window"):SetStyle("IgnoreMouse", true)
				self.wndMain:FindChild("Swap3Window"):SetSprite(nil)
				self.wndMain:FindChild("Swap3Window"):SetTooltip("")
			else
				self.wndMain:FindChild("Swap3Window"):SetStyle("IgnoreMouse", false)
				self.wndMain:FindChild("Swap3Window"):SetSprite("AbilitiesSprites:spr_FrameSelection")
				self.wndMain:FindChild("Swap3Window"):SetTooltip("Swap 3 - Move me!")
			end
		elseif swap == 2 then
			self.wndMain:FindChild("Swap2Window"):SetStyle("Moveable",locked == 0)
			self.wndMain:FindChild("Swap2Window"):SetStyle("Sizable",locked  == 0)
			if locked == 1 then
				self.wndMain:FindChild("Swap2Window"):SetStyle("IgnoreMouse", true)
				self.wndMain:FindChild("Swap2Window"):SetSprite(nil)
				self.wndMain:FindChild("Swap2Window"):SetTooltip("")
			else
				self.wndMain:FindChild("Swap2Window"):SetStyle("IgnoreMouse", false)
				self.wndMain:FindChild("Swap2Window"):SetSprite("AbilitiesSprites:spr_FrameSelection")
				self.wndMain:FindChild("Swap2Window"):SetTooltip("Swap 2 - Move me!")
			end
		else
			self.wndMain:FindChild("Swap1Window"):SetStyle("Moveable",locked == 0)
			self.wndMain:FindChild("Swap1Window"):SetStyle("Sizable",locked  == 0)
			if locked == 1 then
				self.wndMain:FindChild("Swap1Window"):SetStyle("IgnoreMouse", true)
				self.wndMain:FindChild("Swap1Window"):SetSprite(nil)
				self.wndMain:FindChild("Swap1Window"):SetTooltip("")
			else
				self.wndMain:FindChild("Swap1Window"):SetStyle("IgnoreMouse", false)
				self.wndMain:FindChild("Swap1Window"):SetSprite("AbilitiesSprites:spr_FrameSelection")
				self.wndMain:FindChild("Swap1Window"):SetTooltip("Swap 1 - Move me!")
			end
		end
	end

	if self.wndOptions then
		if swap == 4 then
			self.wndOptions:FindChild("Swap4"):FindChild("LockIconButton"):SetCheck(locked == 1)
		elseif swap == 3 then
			self.wndOptions:FindChild("Swap3"):FindChild("LockIconButton"):SetCheck(locked == 1)
		elseif swap == 2 then
			self.wndOptions:FindChild("Swap2"):FindChild("LockIconButton"):SetCheck(locked == 1)
		else
			self.wndOptions:FindChild("Swap1"):FindChild("LockIconButton"):SetCheck(locked == 1)
		end
	end
end

function SpellSwap:SetIconHide(swap, hidden)
	if hidden == 1 then
		if self.wndMain then
			if swap == 4 then
				self.wndMain:FindChild("Swap4Window"):Show(false, true)
			elseif swap == 3 then
				self.wndMain:FindChild("Swap3Window"):Show(false, true)
			elseif swap == 2 then
				self.wndMain:FindChild("Swap2Window"):Show(false, true)
			else
				self.wndMain:FindChild("Swap1Window"):Show(false, true)
			end
		end
	else
		if self.wndMain then
			if swap == 4 then
				self.wndMain:FindChild("Swap4Window"):Show(true, true)
			elseif swap == 3 then
				self.wndMain:FindChild("Swap3Window"):Show(true, true)
			elseif swap == 2 then
				self.wndMain:FindChild("Swap2Window"):Show(true, true)
			else
				self.wndMain:FindChild("Swap1Window"):Show(true, true)
			end
		end
	end
	
	if self.wndOptions then
		if swap == 4 then
			self.wndOptions:FindChild("Swap4"):FindChild("HideIconButton"):SetCheck(hidden == 1)
		elseif swap == 3 then
			self.wndOptions:FindChild("Swap3"):FindChild("HideIconButton"):SetCheck(hidden == 1)
		elseif swap == 2 then
			self.wndOptions:FindChild("Swap2"):FindChild("HideIconButton"):SetCheck(hidden == 1)
		else
			self.wndOptions:FindChild("Swap1"):FindChild("HideIconButton"):SetCheck(hidden == 1)
		end
	end
end

function SpellSwap:SetSpellTextBoxes()
	local spell1,spell2
	
	if self.wndOptions then  
		self.wndOptions:FindChild("Swap1"):FindChild("Spell1TextBox"):SetText(self.vars1.spell1)
		self.wndOptions:FindChild("Swap1"):FindChild("Spell2TextBox"):SetText(self.vars1.spell2)
		
		self.wndOptions:FindChild("Swap2"):FindChild("Spell1TextBox"):SetText(self.vars2.spell1)
		self.wndOptions:FindChild("Swap2"):FindChild("Spell2TextBox"):SetText(self.vars2.spell2)
		
		self.wndOptions:FindChild("Swap3"):FindChild("Spell1TextBox"):SetText(self.vars3.spell1)
		self.wndOptions:FindChild("Swap3"):FindChild("Spell2TextBox"):SetText(self.vars3.spell2)
		
		self.wndOptions:FindChild("Swap4"):FindChild("Spell1TextBox"):SetText(self.vars4.spell1)
		self.wndOptions:FindChild("Swap4"):FindChild("Spell2TextBox"):SetText(self.vars4.spell2)
	end
end

function SpellSwap:SaveSpells()
	local spellName11 = self.wndOptions:FindChild("Swap1"):FindChild("Spell1TextBox"):GetText()
	local spellName12 = self.wndOptions:FindChild("Swap1"):FindChild("Spell2TextBox"):GetText()
	local spellName21 = self.wndOptions:FindChild("Swap2"):FindChild("Spell1TextBox"):GetText()
	local spellName22 = self.wndOptions:FindChild("Swap2"):FindChild("Spell2TextBox"):GetText()
	local spellName31 = self.wndOptions:FindChild("Swap3"):FindChild("Spell1TextBox"):GetText()
	local spellName32 = self.wndOptions:FindChild("Swap3"):FindChild("Spell2TextBox"):GetText()
	local spellName41 = self.wndOptions:FindChild("Swap4"):FindChild("Spell1TextBox"):GetText()
	local spellName42 = self.wndOptions:FindChild("Swap4"):FindChild("Spell2TextBox"):GetText()
	
	local spellId11 = self:GetSpellId(spellName11)
	local spellId12 = self:GetSpellId(spellName12)
	local spellId21 = self:GetSpellId(spellName21)
	local spellId22 = self:GetSpellId(spellName22)
	local spellId31 = self:GetSpellId(spellName31)
	local spellId32 = self:GetSpellId(spellName32)
	local spellId41 = self:GetSpellId(spellName41)
	local spellId42 = self:GetSpellId(spellName42)
	
	-- Swap 1 Section
	if self.vars1.bHide == 0 then
		if spellId11 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName11 .. "' could not be found for Swap 1.")
		elseif spellId12 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName12 .. "' could not be found for Swap 1.")
		elseif spellName11 == spellName12 then
			Print("[WARNING - SpellSwap]: Spells cannot be the same for Swap 1.")
	    else
			self.vars1.spell1 = spellName11
			self.vars1.spell2 = spellName12
			self.vars1.spell1Id = spellId11
			self.vars1.spell2Id = spellId12
			self.vars1.spell1Icon = self:GetSpellIconByName(self.vars1.spell1)
			self.vars1.spell2Icon = self:GetSpellIconByName(self.vars1.spell2)
		end
	end
	
	-- Swap 2 Section
	if self.vars2.bHide == 0 then
		if spellId21 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName21 .. "' could not be found for Swap 2.")
		elseif spellId22 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName22 .. "' could not be found for Swap 2.")
		elseif spellName21 == spellName22 then
			Print("[WARNING - SpellSwap]: Spells cannot be the same for Swap 2.")
	    else
			self.vars2.spell1 = spellName21
			self.vars2.spell2 = spellName22
			self.vars2.spell1Id = spellId21
			self.vars2.spell2Id = spellId22
			self.vars2.spell1Icon = self:GetSpellIconByName(self.vars2.spell1)
			self.vars2.spell2Icon = self:GetSpellIconByName(self.vars2.spell2)
		end
	end
	
	-- Swap 3 Section
	if self.vars3.bHide == 0 then
		if spellId31 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName31 .. "' could not be found for Swap 3.")
		elseif spellId32 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName32 .. "' could not be found for Swap 3.")
		elseif spellName31 == spellName32 then
			Print("[WARNING - SpellSwap]: Spells cannot be the same for Swap 3.")
	    else
			self.vars3.spell1 = spellName31
			self.vars3.spell2 = spellName32
			self.vars3.spell1Id = spellId31
			self.vars3.spell2Id = spellId32
			self.vars3.spell1Icon = self:GetSpellIconByName(self.vars3.spell1)
			self.vars3.spell2Icon = self:GetSpellIconByName(self.vars3.spell2)
		end
	end
	
	-- Swap 4 Section
	if self.vars4.bHide == 0 then
		if spellId41 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName41 .. "' could not be found for Swap 4.")
		elseif spellId42 == nil then
			Print("[WARNING - SpellSwap]: Spell '" .. spellName42 .. "' could not be found for Swap 4.")
		elseif spellName41 == spellName42 then
			Print("[WARNING - SpellSwap]: Spells cannot be the same for Swap 4.")
	    else
			self.vars4.spell1 = spellName41
			self.vars4.spell2 = spellName42
			self.vars4.spell1Id = spellId41
			self.vars4.spell2Id = spellId42
			self.vars4.spell1Icon = self:GetSpellIconByName(self.vars4.spell1)
			self.vars4.spell2Icon = self:GetSpellIconByName(self.vars4.spell2)
		end
	end
	
	self:SkillButtonUpdate(1)
	self:SkillButtonUpdate(2)
	self:SkillButtonUpdate(3)
	self:SkillButtonUpdate(4)
end

---------- Main functions ----------

function SpellSwap:SetDefaults()
    self.vars1 = {}
    self.vars2 = {}
    self.vars3 = {}
    self.vars4 = {}
	
	-- Default Swap 1 values
	self.vars1.spell1 = "Spell 1"
	self.vars1.spell2 = "Spell 2"
	self.vars1.bLock = 0
	self.vars1.bHide = 0
	self:SetIconLock(1, self.vars1.bLock)
	self:SetIconHide(1, self.vars1.bHide)
	self:SetSpellTextBoxes(1)
	
	-- Default Swap 2 values
	self.vars2.spell1 = "Spell 1"
	self.vars2.spell2 = "Spell 2"
	self.vars2.bLock = 0
	self.vars2.bHide = 1
	self:SetIconLock(2, self.vars2.bLock)
	self:SetIconHide(2, self.vars2.bHide)
	self:SetSpellTextBoxes(2)
	
	-- Default Swap 3 values
	self.vars3.spell1 = "Spell 1"
	self.vars3.spell2 = "Spell 2" 
	self.vars3.bLock = 0
	self.vars3.bHide = 1
	self:SetIconLock(3, self.vars3.bLock)
	self:SetIconHide(3, self.vars3.bHide)
	self:SetSpellTextBoxes(3)
	
	-- Default Swap 4 values
	self.vars4.spell1 = "Spell 1"
	self.vars4.spell2 = "Spell 2"
	self.vars4.bLock = 0
	self.vars4.bHide = 1
	self:SetIconLock(4, self.vars4.bLock)
	self:SetIconHide(4, self.vars4.bHide)
	self:SetSpellTextBoxes(4)
end

function SpellSwap:GetAbilitiesList()
	if self.abilitiesList == nil then
		self.abilitiesList = AbilityBook.GetAbilitiesList()
	end
	return self.abilitiesList
end

function SpellSwap:GetSpellId(spellName)
	local abilities = self:GetAbilitiesList()
	if not abilities then
		return
	end

	 for _, v in pairs(abilities) do
 		if v.strName == spellName then
 			return v.nId
 		end
	 end
	
	return nil
end

function SpellSwap:GetSpellIconByName(spellName)
	local iconName
	local abilities = self:GetAbilitiesList()
	if abilities ~= nil then
		for _, ability in pairs(abilities) do
			if ability.strName == spellName then
				return ability.tTiers[1].splObject:GetIcon()
			end
		end
	end
	return ""
end

function SpellSwap:OnWindowMouseDown1(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
	self:ToggleSpellSwap(1)
end

function SpellSwap:OnWindowMouseDown2(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
	self:ToggleSpellSwap(2)
end

function SpellSwap:OnWindowMouseDown3(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
	self:ToggleSpellSwap(3)
end

function SpellSwap:OnWindowMouseDown4(wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation)
	self:ToggleSpellSwap(4)
end

function SpellSwap:OnAbilityChange()
	self:CreateDelayTimer()
end

function SpellSwap:CreateDelayTimer()
	Apollo.CreateTimer("SpellSwap_DelayTimer",1,false)
end

function SpellSwap:OnDelayTimer()
	self:SkillButtonUpdate(1)
	self:SkillButtonUpdate(2)
	self:SkillButtonUpdate(3)
	self:SkillButtonUpdate(4)
end

function SpellSwap:SkillButtonUpdate(swap)
	local las = ActionSetLib.GetCurrentActionSet()
	local newicon,skillsfound,spell1Id,spell2Id,spell1Icon,spell2Icon,w,hidden
	skillsfound = 0
	
	if swap == 4 then
		spell1Id = self.vars4.spell1Id
		spell2Id = self.vars4.spell2Id
		spell1Icon = self.vars4.spell1Icon
		spell2Icon = self.vars4.spell2Icon
		hidden = self.vars4.bHide
		w = self.wndMain:FindChild("Swap4Window")
	elseif swap == 3 then
		spell1Id = self.vars3.spell1Id
		spell2Id = self.vars3.spell2Id
		spell1Icon = self.vars3.spell1Icon
		spell2Icon = self.vars3.spell2Icon
		hidden = self.vars3.bHide
		w = self.wndMain:FindChild("Swap3Window")
	elseif swap == 2 then
		spell1Id = self.vars2.spell1Id
		spell2Id = self.vars2.spell2Id
		spell1Icon = self.vars2.spell1Icon
		spell2Icon = self.vars2.spell2Icon
		hidden = self.vars2.bHide
		w = self.wndMain:FindChild("Swap2Window")
	else 
		spell1Id = self.vars1.spell1Id
		spell2Id = self.vars1.spell2Id
		spell1Icon = self.vars1.spell1Icon
		spell2Icon = self.vars1.spell2Icon
		hidden = self.vars1.bHide
		w = self.wndMain:FindChild("Swap1Window")
	end
	
	if las then
		for k,v in pairs(las) do
			if v == spell1Id then
				newicon= spell2Icon
				skillsfound = skillsfound + 1
			elseif v == spell2Id then
				newicon= spell1Icon
				skillsfound = skillsfound + 1
			end
		end
	else
		self:CreateDelayTimer()
	end
	if skillsfound ~= 1 then
		w:Show(false,false)
		newicon= InactiveSprite
		if hidden == 0 then
			if self.fromReset ~= true then
				if skillsfound > 1 then
					Print("[WARNING - SpellSwap]: Both spells provided for Swap " .. swap .. " are already on your LAS.")
				else
					Print("[WARNING - SpellSwap]: One or more of your configured spells for Swap " .. swap .. " could not be found.")
				end
			else
				self.fromReset = false
			end
		end
	else
		if(hidden == 1) then
			w:Show(false, true)
		else
			w:Show(true, true)
		end
	end
	w:FindChild("SkillButton"):SetSprite(newicon)
end

function SpellSwap:ToggleSpellSwap(swap)
	local luaAbilities = Apollo.GetAddon("Abilities")
	if luaAbilities and luaAbilities.wndMain then
		luaAbilities:OnCloseFinal()
	end
	local las = ActionSetLib.GetCurrentActionSet()
	local oldid,newid,newicon,skillsfnd,laskey,spell1Id,spell2Id,spell1Icon,spell2Icon,w,hidden
	skillsfound = 0
	
	if swap == 4 then
		spell1Id = self.vars4.spell1Id
		spell2Id = self.vars4.spell2Id
		spell1Icon = self.vars4.spell1Icon
		spell2Icon = self.vars4.spell2Icon
		w = self.wndMain:FindChild("Swap4Window")
	elseif swap == 3 then
		spell1Id = self.vars3.spell1Id
		spell2Id = self.vars3.spell2Id
		spell1Icon = self.vars3.spell1Icon
		spell2Icon = self.vars3.spell2Icon
		w = self.wndMain:FindChild("Swap3Window")
	elseif swap == 2 then
		spell1Id = self.vars2.spell1Id
		spell2Id = self.vars2.spell2Id
		spell1Icon = self.vars2.spell1Icon
		spell2Icon = self.vars2.spell2Icon
		w = self.wndMain:FindChild("Swap2Window")
	else 
		spell1Id = self.vars1.spell1Id
		spell2Id = self.vars1.spell2Id
		spell1Icon = self.vars1.spell1Icon
		spell2Icon = self.vars1.spell2Icon
		w = self.wndMain:FindChild("Swap1Window")
	end
	
	for k,v in pairs(las) do
		if v == spell1Id then
			oldid = spell1Id
			newid = spell2Id
			laskey = k
			newicon = spell1Icon
			skillsfound = skillsfound+1
		elseif v == spell2Id then
			oldid = spell2Id
			newid = spell1Id
			laskey = k
			newicon = spell2Icon
			skillsfound = skillsfound+1
		end
	end
	if skillsfound == 1 then
		local abs = AbilityBook.GetAbilitiesList()
		local nTier
		for k,v in pairs(abs) do
			if v.nId == oldid then
				nTier = v.nCurrentTier
				break
			end
		end
		AbilityBook.UpdateSpellTier(oldid,1)
		AbilityBook.UpdateSpellTier(newid,nTier)
		las[laskey] = newid
		local tResult = ActionSetLib.RequestActionSetChanges(las)
		if tResult.eResult == ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
			w:FindChild("SkillButton"):SetSprite(newicon)
		end
	end
end

function SpellSwap:OnSave(eLevel)
	self.vars = {}
	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	self.vars1.l,self.vars1.t,self.vars1.r,self.vars1.b = self.wndMain:FindChild("Swap1Window"):GetAnchorOffsets()
	self.vars2.l,self.vars2.t,self.vars2.r,self.vars2.b = self.wndMain:FindChild("Swap2Window"):GetAnchorOffsets()
	self.vars3.l,self.vars3.t,self.vars3.r,self.vars3.b = self.wndMain:FindChild("Swap3Window"):GetAnchorOffsets()
	self.vars4.l,self.vars4.t,self.vars4.r,self.vars4.b = self.wndMain:FindChild("Swap4Window"):GetAnchorOffsets()
	
	self.vars.one = self.vars1
	self.vars.two = self.vars2
	self.vars.three = self.vars3
	self.vars.four = self.vars4
	return self.vars
end

function SpellSwap:OnRestore(eLevel, tData)
	if tData then
		local pVarNames = {"one","two","three","four"}
		local tVarNames = {"l","t","r","b","bLock","bHide","spell1","spell2","spell1Id","spell2Id","spell1Icon","spell2Icon"}
		
		for _,strVarName in pairs(pVarNames) do
			if tData[strVarName] and strVarName == "one" then
				for k,v in pairs(tVarNames) do
					if tData[strVarName][v] then
						self.vars1[v] = tData[strVarName][v]
					end
				end
			elseif tData[strVarName] and strVarName == "two" then
				for k,v in pairs(tVarNames) do
					if tData[strVarName][v] then
						self.vars2[v] = tData[strVarName][v]
					end
				end
			elseif tData[strVarName] and strVarName == "three" then
				for k,v in pairs(tVarNames) do
					if tData[strVarName][v] then
						self.vars3[v] = tData[strVarName][v]
					end
				end
			elseif tData[strVarName] and strVarName == "four" then
				for k,v in pairs(tVarNames) do
					if tData[strVarName][v] then
						self.vars4[v] = tData[strVarName][v]
					end
				end
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
-- OptionsMenu Functions from interface
---------------------------------------------------------------------------------------------------

-- Lock Icon Event Handlers

function SpellSwap:OnLockIconCheck1(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars1.bLock = 1
	else
		self.vars1.bLock = 0
	end
	self:SetIconLock(1, self.vars1.bLock)
end

function SpellSwap:OnLockIconCheck2(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars2.bLock = 1
	else
		self.vars2.bLock = 0
	end
	self:SetIconLock(2, self.vars2.bLock)
end

function SpellSwap:OnLockIconCheck3(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars3.bLock = 1
	else
		self.vars3.bLock = 0
	end
	self:SetIconLock(3, self.vars3.bLock)
end

function SpellSwap:OnLockIconCheck4(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars4.bLock = 1
	else
		self.vars4.bLock = 0
	end
	self:SetIconLock(4, self.vars4.bLock)
end

-- Hide Icon Event Handlers

function SpellSwap:OnHideIconCheck1(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars1.bHide = 1
	else
		self.vars1.bHide = 0
	end
	self:SetIconHide(1, self.vars1.bHide)
end

function SpellSwap:OnHideIconCheck2(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars2.bHide = 1
	else
		self.vars2.bHide = 0
	end
	self:SetIconHide(2, self.vars2.bHide)
end

function SpellSwap:OnHideIconCheck3(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars3.bHide = 1
	else
		self.vars3.bHide = 0
	end
	self:SetIconHide(3, self.vars3.bHide)
end

function SpellSwap:OnHideIconCheck4(wndHandler, wndControl, eMouseButton)
	if wndControl:IsChecked() then
		self.vars4.bHide = 1
	else
		self.vars4.bHide = 0
	end
	self:SetIconHide(4, self.vars4.bHide)
end

-- Other Event Handlers

function SpellSwap:OnOptionsMenuCloseButton(wndHandler, wndControl, eMouseButton)
	self:CloseOptionsMenu()
end

function SpellSwap:OnSpellSave(wndHandler, wndControl, eMouseButton)
	self:SaveSpells()
end

function SpellSwap:OnSpellSaveAndClose(wndHandler, wndControl, eMouseButton)
	self:SaveSpells()
	self:CloseOptionsMenu()
end

-----------------------------------------------------------------------------------------------
-- SpellSwap Instance
-----------------------------------------------------------------------------------------------
local SpellSwapInst = SpellSwap:new()
SpellSwapInst:Init()
